@@ -77,7 +77,7 @@ enum {
 
 /* This will change if there are incompatible
  * protocol changes */
-#define GDM_GREETER_PROTOCOL_VERSION "1"
+#define GDM_GREETER_PROTOCOL_VERSION "2"
 
 #define GDM_MSG        'D'
 #define GDM_NOECHO     'U'
@@ -107,6 +107,7 @@ enum {
 #define GDM_ERRDLG     'E' /* Puts string up in an error dialog */
 #define GDM_NOFOCUS    'f' /* Don't focus the login window (optional) */
 #define GDM_FOCUS      'F' /* Allow focus on the login window again (optional) */
+#define GDM_SAVEDIE    '!' /* Save wm order and die (and set busy cursor) */
 
 /* Different login interruptions */
 #define GDM_INTERRUPT_TIMED_LOGIN 'T'
@@ -212,7 +213,7 @@ enum {
 #define GDM_KEY_XINERAMASCREEN "greeter/XineramaScreen=0"
 #define GDM_KEY_BACKGROUNDPROG "greeter/BackgroundProgram="
 #define GDM_KEY_BACKGROUNDIMAGE "greeter/BackgroundImage="
-#define GDM_KEY_BACKGROUNDCOLOR "greeter/BackgroundColor=#007777"
+#define GDM_KEY_BACKGROUNDCOLOR "greeter/BackgroundColor=#363047"
 #define GDM_KEY_BACKGROUNDTYPE "greeter/BackgroundType=2"
 #define GDM_KEY_BACKGROUNDSCALETOFIT "greeter/BackgroundScaleToFit=true"
 #define GDM_KEY_BACKGROUNDREMOTEONLYCOLOR "greeter/BackgroundRemoteOnlyColor=true"
@@ -326,6 +327,9 @@ struct _GdmDisplay {
     /* Notification connection */
     int master_notify_fd;  /* write part of the connection */
     int slave_notify_fd; /* read part of the connection */
+
+    /* order in the Xservers file for sessreg, -1 if unset yet */
+    int x_servers_order;
 };
 
 typedef struct _GdmXServer GdmXServer;
@@ -429,6 +433,9 @@ GdmXServer *	gdm_find_x_server	(const char *id);
 #define GDM_SOP_HUP_ALL_GREETERS "HUP_ALL_GREETERS" /* no arguments */
 /* sometimes we can't do a syslog so we tell the main daemon */
 #define GDM_SOP_SYSLOG "SYSLOG" /* <pid> <type> <message> */
+/* write out a sessreg (xdm) compatible Xservers file
+ * in the ServAuthDir as <name>.Xservers */
+#define GDM_SOP_WRITE_X_SERVERS "WRITE_X_SERVERS" /* <slave pid> */
 
 /* Notification protocol */
 #define GDM_NOTIFY_ALLOWREMOTEROOT "AllowRemoteRoot" /* <true/false as int> */
