@@ -1,7 +1,7 @@
 /* Misc routines
  *
  * (c) 2000 Eazel, Inc.
- * (c) 2001 George Lebl
+ * (c) 2001,2002 George Lebl
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,152 +26,33 @@
 
 #include "ve-misc.h"
 
-/**
- * ve_setenv:
- * 
- * Adds "*name=*value" to the environment
- *
- * Returns: 0 on success -1 on error
- * (Note stolen from eel)
- * 
- **/
-int
-ve_setenv (const char *name, const char *value, gboolean overwrite)
-{
-#if defined (HAVE_SETENV)
-	return setenv (name, value, overwrite);
-#else
-	char *string;
-	
-	if (! overwrite && g_getenv (name) != NULL) {
-		return 0;
-	}
-	
-	/* This results in a leak when you overwrite existing
-	 * settings. It would be fairly easy to fix this by keeping
-	 * our own parallel array or hash table.
-	 */
-	string = g_strconcat (name, "=", value, NULL);
-	return putenv (string);
-#endif
-}
-
-/**
- * ve_unsetenv:
- * 
- * Removes *name from the environment
- * (Note stolen from eel)
- * 
- **/
-void
-ve_unsetenv (const char *name)
-{
-#if defined (HAVE_SETENV)
-	unsetenv (name);
-#else
-	extern char **environ;
-	int i, len;
-
-	len = strlen (name);
-	
-	/* Mess directly with the environ array.
-	 * This seems to be the only portable way to do this.
-	 */
-	for (i = 0; environ[i] != NULL; i++) {
-		if (strncmp (environ[i], name, len) == 0
-		    && environ[i][len + 1] == '=') {
-			break;
-		}
-	}
-	while (environ[i] != NULL) {
-		environ[i] = environ[i + 1];
-		i++;
-	}
-#endif
-}
-
-void
-ve_clearenv (void)
-{
-#ifdef HAVE_CLEARENV
-	clearenv ();
-#else
-	extern char **environ;
-	environ[0] = NULL;
-#endif
-}
-
-/* Do strcasecmp but ignore locale */
-int
-ve_strcasecmp_no_locale (const char *s1, const char *s2)
-{
-	int i;
-
-	/* Error, but don't make them equal then */
-	g_return_val_if_fail (s1 != NULL, G_MAXINT);
-	g_return_val_if_fail (s2 != NULL, G_MININT);
-
-	for (i = 0; s1[i] != '\0' && s2[i] != '\0'; i++) {
-		char a = s1[i];
-		char b = s2[i];
-
-		if (a >= 'A' && a <= 'Z')
-			a -= 'A' - 'a';
-		if (b >= 'A' && b <= 'Z')
-			b -= 'A' - 'a';
-
-		if (a < b)
-			return -1;
-		else if (a > b)
-			return 1;
-	}
-
-	/* find out which string is smaller */
-	if (s2[i] != '\0')
-		return -1; /* s1 is smaller */
-	else if (s1[i] != '\0')
-		return 1; /* s2 is smaller */
-	else
-		return 0; /* equal */
-}
-
 char **
 ve_split (const char *s)
 {
 	int argc;
-	char **temp_argv;
-	char **ret;
-	int i;
+	char **argv;
 
 	if (s == NULL)
 		return NULL;
 
-	if (poptParseArgvString (s, &argc, &temp_argv) != 0) {
+	if (g_shell_parse_argv (s, &argc, &argv, NULL)) {
+		return argv;
+	} else {
 		return g_strsplit (s, " ", -1);
 	}
-
-	ret = g_new (char *, argc+1);
-	for (i = 0; i < argc; i++) {
-		ret[i] = g_strdup (temp_argv[i]);
-	}
-	ret[i] = NULL;
-
-	free (temp_argv);
-
-	return ret;
 }
 
 char *
 ve_first_word (const char *s)
 {
 	int argc;
-	char **temp_argv;
+	char **argv;
 	char *ret;
 
 	if (s == NULL)
 		return NULL;
 
-	if (poptParseArgvString (s, &argc, &temp_argv) != 0) {
+	if ( ! g_shell_parse_argv (s, &argc, &argv, NULL)) {
 		char *p;
 		ret = g_strdup (s);
 		p = strchr (ret, ' ');
@@ -180,9 +61,9 @@ ve_first_word (const char *s)
 		return ret;
 	}
 
-	ret = g_strdup (temp_argv[0]);
+	ret = g_strdup (argv[0]);
 
-	free (temp_argv);
+	g_strfreev (argv);
 
 	return ret;
 }
@@ -260,6 +141,15 @@ ve_vector_len (char * const *v)
 	return i;
 }
 
+gboolean
+ve_bool_equal (gboolean a, gboolean b)
+{
+	if ((a && b) || (!a && !b))
+		return TRUE;
+	else
+		return FALSE;
+}
+
 gboolean
 ve_is_string_in_list (const GList *list, const char *string)
 {
@@ -283,7 +173,7 @@ ve_is_string_in_list_case_no_locale (const GList *list, const char *string)
 
 	while (list != NULL) {
 		if (list->data != NULL &&
-		    ve_strcasecmp_no_locale (string, list->data) == 0)
+		    g_ascii_strcasecmp (string, list->data) == 0)
 			return TRUE;
 
 		list = list->next;
@@ -295,7 +185,8 @@ ve_is_string_in_list_case_no_locale (const GList *list, const char *string)
 char *
 ve_find_file (const char *filename, const GList *directories)
 {
-	char *s, *ss, *path;
+	char *s, *ss;
+	const char *path;
 	char **pathv;
 	int i;
 	const GList *li;
@@ -311,18 +202,32 @@ ve_find_file (const char *filename, const GList *directories)
 		return NULL;
 
 	for (li = directories; li != NULL; li = li->next) {
-		s = g_concat_dir_and_file (li->data, filename);
+		s = g_build_filename (li->data, filename, NULL);
 		if (access (s, F_OK) == 0)
 			return s;
 		g_free (s);
 	}
 
-	s = gnome_datadir_file (filename);
+	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+				       filename, TRUE, NULL);
 	if (s != NULL)
 		return s;
 
-	ss = g_concat_dir_and_file (g_get_prgname (), filename);
-	s = gnome_datadir_file (ss);
+	ss = g_build_filename (g_get_prgname (), filename, NULL);
+	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+				       ss, TRUE, NULL);
+	g_free (ss);
+	if (s != NULL)
+		return s;
+
+	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR,
+				       filename, TRUE, NULL);
+	if (s != NULL)
+		return s;
+
+	ss = g_build_filename (g_get_prgname (), filename, NULL);
+	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR,
+				       ss, TRUE, NULL);
 	g_free (ss);
 	if (s != NULL)
 		return s;
@@ -331,15 +236,15 @@ ve_find_file (const char *filename, const GList *directories)
 	if (path != NULL) {
 		pathv = g_strsplit (path, ":", 0);
 		for (i = 0; pathv[i] != NULL; i++) {
-			s = g_strconcat (pathv[i], "/share/", filename, NULL);
+			s = g_build_filename (pathv[i], "share", filename, NULL);
 			if (access (s, F_OK) == 0) {
 				g_strfreev (pathv);
 				return s;
 			}
 			g_free (s);
-			s = g_strconcat (pathv[i], "/share/",
-					 g_get_prgname (), "/",
-					filename, NULL);
+			s = g_build_filename (pathv[i], "share",
+					      g_get_prgname (),
+					      filename, NULL);
 			if (access (s, F_OK) == 0) {
 				g_strfreev (pathv);
 				return s;
@@ -351,3 +256,59 @@ ve_find_file (const char *filename, const GList *directories)
 
 	return NULL;
 }
+
+char *
+ve_locale_to_utf8 (const char *str)
+{
+	char *ret = g_locale_to_utf8 (str, -1, NULL, NULL, NULL);
+	if (ret == NULL) {
+		g_warning ("string not in proper locale encoding: \"%s\"", str);
+		return g_strdup (str);
+	} else {
+		return ret;
+	}
+}
+
+char *
+ve_locale_from_utf8 (const char *str)
+{
+	char *ret = g_locale_from_utf8 (str, -1, NULL, NULL, NULL);
+	if (ret == NULL) {
+		g_warning ("string not in proper utf8 encoding: \"%s\"", str);
+		return g_strdup (str);
+	} else {
+		return ret;
+	}
+}
+
+char *
+ve_strftime (struct tm *the_tm, const char *format)
+{
+	char str[1024];
+	char *loc_format = ve_locale_from_utf8 (format);
+
+	if (strftime (str, sizeof (str)-1, loc_format, the_tm) == 0) {
+		/* according to docs, if the string does not fit, the
+		 * contents of str are undefined, thus just use
+		 * ??? */
+		strcpy (str, "???");
+	}
+	str [sizeof (str)-1] = '\0'; /* just for sanity */
+	g_free (loc_format);
+
+	return ve_locale_to_utf8 (str);
+}
+
+pid_t
+ve_waitpid_no_signal (pid_t pid, int *status, int options)
+{
+	pid_t ret;
+
+	for (;;) {
+		ret = waitpid (pid, status, options);
+		if (ret == 0)
+			return 0;
+		if (errno != EINTR)
+			return ret;
+	}
+}
